public class GrowingTree
{
  /** the root node of this tree */
  private TreeNode root;

  public GrowingTree(TreeNode t) { root = t; }

  // Part (a)

  /** Creates two new nodes that each contain the same value as contained in t.
   *  These new nodes become the left and right child nodes of t.
   *  The original left child of t becomes the left child of the new left child,
   *  and the original right child of t becomes the right child of the new right child.
   *  @param t a reference to the node to be expanded
   *          Precondition: t is not null
   */
  private static void expandNode(TreeNode t)
  {
    TreeNode left = new TreeNode(t.getValue(), t.getLeft(), null);
    TreeNode right = new TreeNode(t.getValue(), null, t.getRight());
    t.setLeft(left);
    t.setRight(right);
  }

  // *************************************************

  /** Grows the tree by expanding all nodes that contain the value val
   *  @param val the value to be matched
   */
  public void growTree(Object val)
  { growTreeHelper(root, val); }

  // Part (b)

  /** Grows the tree by expanding all nodes containing val in the subtree rooted at current
   *  @param current the root of the subtree to be processed
   *  @param val the value to be matched
   */
  private static void growTreeHelper(TreeNode current, Object val)
  {
    if (current != null)
    {
      growTreeHelper(current.getLeft(), val);
      growTreeHelper(current.getRight(),val);
      if (current.getValue().equals(val))
        expandNode(current);
    }
  }

  // *************************************************

  // Part (c)

  //  Method                 Big-O
  // ----------------------------------
  //  expandNode              O(1)
  //  growTreeHelper          O(n)

  // *************************************************

  public static void traverseInOrder(TreeNode root)
  {
    if (root != null)
    {
      traverseInOrder(root.getLeft());
      System.out.print(root.getValue() + " ");
      traverseInOrder(root.getRight());
    }
  }

  public static void main(String[] args)
  {
    /*
            x
           / \
          x   y
           \
            w
    */

    TreeNode w = new TreeNode("w", null, null);
    TreeNode x = new TreeNode("x", null, w);
    TreeNode y = new TreeNode("y", null, null);
    TreeNode x0 = new TreeNode("x", x, y);

    GrowingTree g = new GrowingTree(x0);
    traverseInOrder(g.root);
    System.out.println();

    g.growTree("x");
    traverseInOrder(g.root);
    System.out.println();
  }
}