import java.util.Set;
import java.util.TreeSet;

public class Person
  implements Comparable<Person> // added for testing with TreeSet
{
  private String name;
  private Set<Person> contacts;

  public Person(String nm)
  {
    name = nm;
    contacts = new TreeSet<Person>();
  }

  public void addContact(Person p)
  {
    contacts.add(p);
  }

  public int compareTo(Person other)
  {
    return name.compareTo(other.name);
  }

  public String toString()
  {
    return name;
  }

  /** @return a set containing all of this person's immediate contacts (distance 1)
   */
  public Set<Person> getContacts()
  {
    Set<Person> c = new TreeSet<Person>(); // or new HashSet<Person>

    for (Person p : contacts)
      c.add(p);

    return c;
  }

  // Part (a)

  /** @param people a set of Person objects
   *  @return a set containing all members of people and all the
   *          immediate contacts of all members of people
   *  Postcondition: the set people is unchanged
   */
  private static Set<Person> expandContactSet(Set<Person> people)
  {
    Set<Person> c = new TreeSet<Person>(); // or new HashSet<Person>
    for (Person p : people)
    {
      c.add(p);
      for (Person p1 : p.getContacts())
        c.add(p1);
    }
    return c;
  }

  // *************************************************

  // Part (b)

  /** Creates and returns a set containing all members of this person's network of contacts
   *  who are within distance dist away
   *  @param dist the maximum distance between this person and a network contact
   *         Precondition: dist > 0
   *  @return a set of contacts who are within distance dist away from this person
   *         Postcondition: this person is not included in the returned set
   */
  public Set<Person> getNetwork(int dist)
  {
    Set<Person> people = getContacts();

    for (int d = 2; d <= dist; d++)
      people = expandContactSet(people);

    people.remove(this);
    return people;
  }

  // *************************************************

  public static void main(String[] args)
  {
    Person ahmed = new Person("Ahmed");
    Person bonita = new Person("Bonita");
    Person chad = new Person("Chad");
    Person danielle = new Person("Danielle");
    Person eric = new Person("Eric");
    Person fay = new Person("Fay");

    ahmed.addContact(bonita);
    ahmed.addContact(eric);
    bonita.addContact(ahmed);
    bonita.addContact(chad);
    chad.addContact(bonita);
    danielle.addContact(eric);
    danielle.addContact(fay);
    danielle.addContact(fay);
    eric.addContact(ahmed);
    eric.addContact(danielle);
    eric.addContact(fay);
    fay.addContact(danielle);
    fay.addContact(eric);

    System.out.println("Distance = 1");
    System.out.println(ahmed + ": " + ahmed.getNetwork(1));
    System.out.println(bonita + ": " + bonita.getNetwork(1));
    System.out.println(chad + ": " + chad.getNetwork(1));
    System.out.println(danielle + ": " + danielle.getNetwork(1));
    System.out.println(eric + ": " + eric.getNetwork(1));
    System.out.println(fay + ": " + fay.getNetwork(1));
    System.out.println();

    System.out.println("Distance = 2");
    System.out.println(ahmed + ": " + ahmed.getNetwork(2));
    System.out.println(bonita + ": " + bonita.getNetwork(2));
    System.out.println(chad + ": " + chad.getNetwork(2));
    System.out.println(danielle + ": " + danielle.getNetwork(2));
    System.out.println(eric + ": " + eric.getNetwork(2));
    System.out.println(fay + ": " + fay.getNetwork(2));
    System.out.println();

    System.out.println("Distance = 3");
    System.out.println(ahmed + ": " + ahmed.getNetwork(3));
    System.out.println(bonita + ": " + bonita.getNetwork(3));
    System.out.println(chad + ": " + chad.getNetwork(3));
    System.out.println(danielle + ": " + danielle.getNetwork(3));
    System.out.println(eric + ": " + eric.getNetwork(3));
    System.out.println(fay + ": " + fay.getNetwork(3));
    System.out.println();
  }
}